@@ -1,7 +1,7 @@
 /*
- $Id: unicode.h 15571 2013-01-07 14:54:28Z sloot $
+ $Id: unicode.h 17993 2015-01-21 14:51:26Z sloot $
  $URL: https://ilk.uvt.nl/svn/sources/ucto/trunk/include/ucto/unicode.h $
-  Copyright (c) 1998 - 2013
+  Copyright (c) 1998 - 2015
   ILK  -  Tilburg University
   CNTS -  University of Antwerp
  
@@ -53,9 +53,12 @@ namespace Tokenizer {
     friend std::ostream& operator<<( std::ostream&, const UnicodeFilter& );
   public:
     UnicodeString filter( const UnicodeString& );
-    void add( UChar uc, const UnicodeString& us ) { the_map[uc] = us; };
+    bool fill( const std::string& );
+    bool add( const UnicodeString& );
+    bool add( const std::string& );
     bool empty() const { return the_map.empty(); };
   private:
+    void add( UChar uc, const UnicodeString& us ) { the_map[uc] = us; };
     std::map<UChar, UnicodeString> the_map;
   };
   
