@@ -1,10 +1,10 @@
 /*
-  $Id: unicode.cxx 15910 2013-04-03 13:57:51Z sloot $
+  $Id: unicode.cxx 18006 2015-01-29 10:10:59Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/ucto/trunk/src/unicode.cxx $
-  Copyright (c) 1998 - 2013
+  Copyright (c) 1998 - 2015
   ILK  -  Tilburg University
   CNTS -  University of Antwerp
- 
+
   This file is part of Ucto
 
   Ucto is free software; you can redistribute it and/or modify
@@ -28,6 +28,7 @@
 
 #include <string>
 #include <ostream>
+#include <fstream>
 #include <map>
 #include <stdexcept>
 #include "libfolia/foliautils.h"
@@ -53,7 +54,7 @@ namespace Tokenizer {
       throw std::logic_error( "invalid normalization mode: " + enc );
     return mode;
   }
-  
+
   inline string toString( UNormalizationMode mode ){
     switch ( mode ){
     case UNORM_NONE:
@@ -70,7 +71,7 @@ namespace Tokenizer {
       throw std::logic_error( "invalid normalization mode in switch" );
     }
   }
-  
+
   std::string UnicodeNormalizer::getMode( ) const {
     return toString( mode );
   }
@@ -121,4 +122,52 @@ namespace Tokenizer {
     }
   }
 
+  bool UnicodeFilter::add( const string& s ){
+    UnicodeString line = folia::UTF8ToUnicode(s);
+    return add( line );
+  }
+
+  bool UnicodeFilter::add( const UnicodeString& s ){
+    UnicodeString line = s;
+    line.trim();
+    if ((line.length() > 0) && (line[0] != '#')) {
+      UnicodeString open = "";
+      UnicodeString close = "";
+      int splitpoint = line.indexOf(" ");
+      if ( splitpoint == -1 )
+	splitpoint = line.indexOf("\t");
+      if ( splitpoint == -1 ){
+	open = line;
+      }
+      else {
+	open = UnicodeString( line, 0,splitpoint);
+	close = UnicodeString( line, splitpoint+1);
+      }
+      open = open.trim().unescape();
+      close = close.trim().unescape();
+      if ( open.length() != 1 ){
+	throw runtime_error( "invalid filter entry: "
+			     + folia::UnicodeToUTF8(line) );
+      }
+      else {
+	this->add( open[0], close );
+      }
+    }
+    return true;
+  }
+
+  bool UnicodeFilter::fill( const string& s ){
+    ifstream f ( s.c_str() );
+    if ( !f ){
+      throw std::runtime_error("unable to open file: " + s );
+    }
+    else {
+      string rawline;
+      while ( getline(f,rawline) ){
+	this->add( rawline );
+      }
+    }
+    return true;
+  }
+
 } // namespace Tokenizer
